#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright: Damien Elmes <anki@ichi2.net>
# License: GNU GPL, version 2 or later; http://www.gnu.org/copyleft/gpl.html

"""\
Cards
====================
"""
__docformat__ = 'restructuredtext'

import time, random, sys
from anki.ids import *

PRIORITY_HIGH = 4
PRIORITY_MED = 3
PRIORITY_NORM = 2
PRIORITY_LOW = 1
PRIORITY_NONE = 0

class Card(IDObject):
    "A card."

    def __init__(self, fact=None, cardModel=None, tags=None):
        IDObject.__init__(self)
        # scheduling information
        self.priority = PRIORITY_NORM
        if fact:
            self.id = fact.id
        self.fact = fact
        self.model = cardModel
        self.history = []
        if tags:
            self.tags = tags
        else:
            self.tags = []
        if fact:
            self.fact.cards.append(self)

    question = property(lambda self: self.model.renderQuestion(self, self.fact))
    answer = property(lambda self: self.model.renderAnswer(self, self.fact))

    htmlQuestion = property(lambda self: self.model.renderQuestion(
        self, self.fact, type="html"))
    htmlAnswer = property(lambda self: self.model.renderAnswer(
        self, self.fact, type="html"))

    def css(self):
        return self.model.css() + self.fact.css()

    def hasTag(self, tag):
        "True if tag is in this card, fact or model."
        tag = tag.lower()
        return (tag in [t.lower() for t in self.tags] or
                self.fact.hasTag(tag) or
                tag == self.model.name.lower())

    def updatePriority(self):
        "Check tags and update priority. Don't dirty."
        # hack
        deck = self.parent
        for tag in deck.sched.suspendedTags:
            if self.hasTag(tag):
                self.__dict__['priority'] = PRIORITY_NONE
                return
        for tag in deck.sched.highPriority:
            if self.hasTag(tag):
                self.__dict__['priority'] = PRIORITY_HIGH
                return
        for tag in deck.sched.medPriority:
            if self.hasTag(tag):
                self.__dict__['priority'] = PRIORITY_MED
                return
        for tag in deck.sched.lowPriority:
            if self.hasTag(tag):
                self.__dict__['priority'] = PRIORITY_LOW
                return
        self.__dict__['priority'] = PRIORITY_NORM

    def __setattr__(self, k, v):
        self.__dict__[k] = v
        self.setModified()

    def __unicode__(self):
        return ("%s : %s" %
                (self.question, self.answer)).replace("\n", "/")
