#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright: Damien Elmes <anki@ichi2.net>
# License: GNU GPL, version 2 or later; http://www.gnu.org/copyleft/gpl.html

"""\
A simple server for Emacs
==============================
"""
__docformat__ = 'restructuredtext'

import copy
import os, sys
import traceback
import anki.deck

raise "not yet implemented"

class Server(object):

    def __init__(self):
        self.currentDeck = None
        self.deckStorage = anki.deck.Controller.DeckStorage()

    def setDeck(self, args):
        if self.currentDeck and self.currentDeck.modified():
            if not args.get("force", 0):
                return self.writeOutput(status="unsavedChanges")
        (head, tail) = os.path.splitext(args['deck'])
        if tail != ".fc":
            # we're able to read .deck files, but we want the user to
            # convert manually to avoid issues of existing files being
            # overwritten, etc.
            return self.writeOutput(status="invalidFileFormat")
        try:
            deck = os.path.expanduser(args['deck'])
            self.currentDeck = self.deckStorage.load(deck)
        except:
            return self.writeOutput(status="errorLoadingDeck")
        self.writeOutput(status="OK")

    def createDeck(self, args):
        deckPath = args['path']
        (head, tail) = os.path.splitext(deckPath)
        if tail != ".fc":
            return self.writeOutput(status="invalidFileName")
        if os.path.exists(deckPath):
            self.writeOutput(status="fileExists")
        # don't validate the filename, that's up to the frontend
        self.currentDeck = anki.deck.Controller.Deck()
        self.deckStorage.save(self.currentDeck, deckPath)
        self.writeOutput(status="OK")

    def getCard(self, args):
        if self.noCurrentDeck(): return
        card = self.currentDeck.getCard()
        if not card:
            if not self.currentDeck:
                return self.writeOutput(status="deckEmpty")
            else:
                next=self.currentDeck.nextCardMsg()
                return self.writeOutput(status="noCardsAvailable",
                                        nextCard=next)
        else:
            int2 = self.currentDeck.nextIntervalStr(card, 2)
            int3 = self.currentDeck.nextIntervalStr(card, 3)
            int4 = self.currentDeck.nextIntervalStr(card, 4)
            self.writeOutput(status="OK",
                             question=card.question,
                             answer=card.answer,
                             id=card.id,
                             int2=int2,
                             int3=int3,
                             int4=int4)

    def answerCard(self, args):
        if self.noCurrentDeck(): return
        res = self.currentDeck.cardAnswered(
            self.currentDeck.ids[int(args['id'])],
            int(args['quality']))
        self.writeOutput(status="OK",
                         days=res)

    def getStats(self, args):
        if self.noCurrentDeck(): return
        stats = self.currentDeck.getStats()
        stats['status'] = "OK"
        self.writeOutput(**stats)

    def addCards(self, args):
        if self.noCurrentDeck(): return
        lang = args['lang']
        q = args['question']
        a = args['answer']
        if self.currentDeck.hasQuestion(q):
            return self.writeOutput(status="sameQuestionExists")
        if lang == "ja":
            from anki.cardgen import JapaneseCardGenerator
            cards = JapaneseCardGenerator().makeCards(q, a)
        elif lang == "en":
            from anki.cardgen import CardGenerator
            cards = CardGenerator().makeCards(q, a)
        else:
            return self.writeOutput(status="Invalid language")

        for card in cards:
            self.currentDeck.addCard(card)

        self.writeOutput(status="OK",
                         question=cards[0].question,
                         answer=cards[0].answer)

    def editCard(self, args):
        if self.noCurrentDeck(): return
        card = self.currentDeck.ids[int(args['id'])]
        card.question = args['question']
        card.answer = args['answer']
        self.writeOutput(status="OK")

    def deleteCard(self, args):
        if self.noCurrentDeck(): return
        card = self.currentDeck.ids[int(args['id'])]
        self.currentDeck.remove(card)
        self.writeOutput(status="OK")

    def saveDeck(self, args):
        if self.noCurrentDeck(): return
        self.deckStorage.save(self.currentDeck)
        self.writeOutput(status="OK")

    # this is hacky, and on jlpt-2.common results in in sending about
    # 1MB of data to emacs. we can optimise later..
    def getCards(self, args):
        if self.noCurrentDeck(): return
        cards={}
        deck = copy.copy(self.currentDeck)
        deck.sort(key=lambda x: getattr(x, "question"))
        print "((status . \"OK\") ",
        for card in deck:
            self.writeOutputNl(question=card.question,
                               answer=card.answer,
                               id=card.id,
                               status="card")
        print ")"

    def syncDeck(self, args):
        # FIXME: support one-way syncing
        if self.noCurrentDeck(): return
        username=args["username"]
        password=args["password"]
        if not username:
            return self.writeOutput(status="noUsername")
        deckname=os.path.basename(self.currentDeck.path)
        from anki.sync import HttpSyncer
        sync=HttpSyncer(username, password, deckname)
        res=sync.checkAuth()
        if type(res) != type(()):
            return self.writeOutput(status=res)
        stats=sync.sync(self.currentDeck)
        self.deckStorage.save(self.currentDeck)
        self.writeOutput(status="OK",
                         locallyChanged=stats[0],
                         locallyDeleted=stats[1],
                         remotelyChanged=stats[2],
                         remotelyDeleted=stats[3])

    def noCurrentDeck(self):
        if self.currentDeck != None:
            return False
        else:
            self.writeOutput(status="noCurrentDeck")
            return True

    def deckModified(self, args):
        if not self.currentDeck or \
           not self.currentDeck.modified():
            self.writeOutput(status="notModified")
        else:
            self.writeOutput(status="modified")

    def mainLoop(self):
        while 1:
            line = raw_input()
            (cmd, args) = self.parseCmd(line)
            if not cmd:
                continue
            cmd = getattr(self, cmd, None)
            if not cmd:
                self.writeOutput(status="invalidCommand")
                continue
            try:
                cmd(args)
            except:
                type, value, tb = sys.exc_info()
                list = traceback.format_tb(tb, None) + \
                       traceback.format_exception_only(type, value)
                s=", ".join(list)
                s=s.replace("\n", ", ")
                self.writeOutput(status="unhandledException",
                                 exception=s)

    def parseCmd(self, line):
        try:
            (cmd, args) = line.split(" ", 1)
        except ValueError:
            return (line, "")
        try:
            args = eval(args)
        except:
            self.writeOutput(status="invalidDataError")
            return (None, None)
        # convert to unicode - keys should be ascii
        for (k,v) in args.items():
            args[k] = unicode(v, "utf-8")
        return (cmd, args)

    def writeOutput(self, **dict):
        s=self._dictToList(dict)
        print s.encode("utf-8")

    def writeOutputNl(self, **dict):
        """Don't include trailing newline.
        This allows us to output sublists."""
        s=self._dictToList(dict)
        print s.encode("utf-8"),

    def _dictToList(self, dict):
        return "(" + \
               " ".join(['(%s . "%s")' % \
                         (k, unicode(v).replace('"', '\\"')) \
                         for (k, v) in dict.items()]) + ")"

if __name__ == "__main__":
    Server().mainLoop()
