#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright: Damien Elmes <anki@ichi2.net>
# License: GNU GPL, version 2 or later; http://www.gnu.org/copyleft/gpl.html

"""\
Fields - define individual components in a fact & model
==============================================================
"""

import time
from anki.ids import *
from anki.fonts import toPlatform

class Fields(IDList):

    def __unicode__(self):
        return u"<fields size=%d>" % len(self)

class Field(IDObject):
    "The definition of one field in a fact. Call setModified when changing."

    def __init__(self, name="", description=""):
        IDObject.__init__(self)
        self.name = name
        self.description = description
        self.features = []
        self.display = {
            "quiz": {
            "fontFamily": None,
            "fontSize": None,
            "fontColour": None,
            },
            "edit": {
            "fontFamily": None,
            "fontSize": 20,
            },
            }

    def css(self, type="quiz"):
        t = ".%s { " % self.name.replace(" ", "")
        if self.display[type]['fontFamily']:
            t += "font-family: \"%s\"; " % toPlatform(self.display[type]['fontFamily'])
        if self.display[type]['fontSize']:
            t += "font-size: %dpx; " % self.display[type]['fontSize']
        if type == "quiz" and self.display[type]['fontColour']:
            t += "color: %s; " % self.display[type]['fontColour']
        t += " }\n"
        return t

    def __unicode__(self):
        return unicode(self.name)
