#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright: Damien Elmes <anki@ichi2.net>
# License: GNU GPL, version 2 or later; http://www.gnu.org/copyleft/gpl.html

"""\
IDList - a modification tracking, indexed list of objects
==============================================================
"""

import time

class IDList(list):

    def __init__(self, parent):
        self.created = time.time()
        self.modified = self.created
        self.parent = parent
        self.ids = {}

    def append(self, obj):
        self.insert(len(self), obj)

    def insert(self, index, obj):
        obj.parent = self
        self.uniquifyID(obj)
        self.ids[obj.id] = obj
        list.insert(self, index, obj)
        obj.setModified()

    def setModified(self, newTime=None):
        if not newTime:
            newTime = time.time()
        self.__dict__['modified'] = newTime
        if self.__dict__.get('parent', None) is not None:
            self.parent.setModified(newTime)

    def __delitem__(self, index):
        self.ids[self[index].id] = None
        list.__delitem__(self, index)
        self.setModified()

    def remove(self, obj):
        list.remove(self, obj)
        self.ids[obj.id] = None
        self.setModified()

    def uniquifyID(self, obj):
        while obj.id in self.ids:
            if self.ids[obj.id] is None:
                # reuse the deleted ID
                return
            # this will keep the object within int boundaries
            obj.id = hash(obj.id + 1)

    def __repr__(self):
        return self.__unicode__()

class IDObject(object):

    def __init__(self):
        self.created = time.time()
        self.modified = self.created
        self.id = hash(self.created)
        self.parent = None

    def __eq__(self, other):
        if not isinstance(other, IDObject):
            return False
        if self.id == other.id:
            return True
        return False

    def setModified(self, newTime=None):
        if not newTime:
            newTime = time.time()
        self.__dict__['modified'] = newTime
        if self.__dict__.get('parent', None) is not None:
            self.parent.setModified(newTime)

    def __repr__(self):
        return `self.__unicode__()`
