#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright: Damien Elmes <anki@ichi2.net>
# License: GNU GPL, version 2 or later; http://www.gnu.org/copyleft/gpl.html

import sys, os, pickle
from anki.decorators import Decorator, Decorators
from anki.utils import findTag, stripHTML

class UnihanController(object):

    def __init__(self, target):
        base = os.path.dirname(os.path.abspath(__file__))
        if not os.path.exists(base):
            if sys.platform.startswith("darwin"):
                base = os.path.dirname(sys.argv[0])
            else:
                base = os.path.join(os.path.dirname(sys.argv[0]),
                                    "support")
        db = os.path.join(base, "unihan.pickle")
        self.db = pickle.load(open(db, "rb"))
        self.type = target

    def reading(self, text):
        text = stripHTML(text)
        result = []
        for c in text:
            n = ord(c)
            if (n in self.db and
                self.db[n].has_key(self.type)):
                result.append(self.formatMatch(self.db[n][self.type]))
        return " ".join(result)

    def formatMatch(self, match):
        m = match.split(" ")
        if len(m) == 1:
            return m[0]
        return "{%s}" % (",".join(m))

class ChineseGenerator(Decorator):

    def __init__(self):
        self.expressionField = "Expression"
        self.readingField = "Reading"

    def lazyInit(self):
        pass

    def onKeypress(self, fact, field, value):
        if findTag("Reading source", field.features):
            dst = None
            for field in fact.model.fields:
                if findTag("Reading destination", field.features):
                    dst = field
                    break
            if not dst:
                return
            self.lazyInit()
            reading = self.unihan.reading(value)
            fact[dst.name] = reading

class CantoneseGenerator(ChineseGenerator):

    def __init__(self):
        ChineseGenerator.__init__(self)
        self.tags = ["Cantonese"]
        self.name = "Reading generation for Cantonese"

    def lazyInit(self):
        if 'unihan' not in self.__dict__:
            self.unihan = UnihanController("cantonese")

class MandarinGenerator(ChineseGenerator):

    def __init__(self):
        ChineseGenerator.__init__(self)
        self.tags = ["Mandarin"]
        self.name = "Reading generation for Mandarin"

    def lazyInit(self):
        if 'unihan' not in self.__dict__:
            self.unihan = UnihanController("mandarin")

def install():
    Decorators.add(CantoneseGenerator())
    Decorators.add(MandarinGenerator())
