# coding: utf-8

import nose, os
import anki
from anki.errors import *
from anki.facts import *
from anki.models import *
from anki.importing import *
from anki import stdmodels

deck = None
ds = None
testDir = os.path.dirname(__file__)

def setup1():
    global deck, ds
    deck = anki.Controller.Deck()
    ds = anki.Controller.DeckStorage()

def assertException(exception, func):
    found = False
    try:
        func()
    except exception:
        found = True
    assert found

##########################################################################

@nose.with_setup(setup1)
def test_importoldv7jp():
    # however, if we give it the correct model, it should work
    file = unicode(os.path.join(testDir, "importing/test-v8-jp.fc"))
    model = stdmodels.byName("Japanese")
    AnkiV2Importer(file, model).importInto(deck)
    assert len(deck) == 7
    # make sure we've assigned the scheduling data correctly
    card = [c for c in deck if c.question == u"飲む"][0]
    assert card.interval > 5
    card = [c for c in deck if c.answer == u"飲む<br>のむ"][0]
    assert card.interval == 0

def test_importsampledecks():
    deckDir = os.path.join(testDir, "../anki/samples")
    deckDir = "/home/resolve/decks"
    if not os.path.exists(deckDir):
        print "ignoring..",
        return
    #deckDir = "/home/resolve/samples"
    return
    decks = os.listdir(unicode(deckDir))
    for deckname in decks:
        if "jlpt" in deckname:
            continue
        if deckname.endswith(".fc"):
            print "testing", deckname, "as jp"
            setup1()
            file = os.path.join(deckDir, deckname)
            model = stdmodels.byName("Japanese")
            try:
                AnkiV2Importer(file, model).importInto(deck)
            except DeckWrongFormatError:
                continue
            print "testing", deckname, "as en"
            setup1()
            file = os.path.join(deckDir, deckname)
            model = stdmodels.byName("English")
            AnkiV2Importer(file, model).importInto(deck)

@nose.with_setup(setup1)
def test_importmnemosyne():
    global deck
    file = os.path.join(testDir, "importing/test.mem")
    model = stdmodels.byName("Other")
    rep = MnemosyneImporter(file, model).importInto(deck)
    assert len(deck) == 5
    assert rep.dupes == 0
    # test correct scheduling assignment
    c = [c for c in deck if c.question == 'baz'][0]
    assert c.interval == 1
    c = [c for c in deck if c.question == 'quux'][0]
    assert c.interval == 0
    setup1()
    model = stdmodels.byName("Japanese")
    imp = MnemosyneImporter(file, model)
    imp.importInto(deck)
    # any missing cards are added
    assert len(deck) == 6
    c = [c for c in deck if c.question == 'baz'][0]
    assert c.interval == 1
    c = [c for c in deck if c.question == 'quux'][0]
    assert c.interval == 0
    # now let's try again, removing one of the models
    setup1()
    model = stdmodels.byName("Japanese")
    model.cards = [model.cards[0]]
    imp = MnemosyneImporter(file, model)
    imp.importInto(deck)
    # only five, no extras were added
    assert len(deck) == 5

@nose.with_setup(setup1)
def test_importtext():
    file = os.path.join(testDir, "importing/text-2fields.txt")
    model = stdmodels.byName("Other")
    model.cards = model.allcards
    rep = TextImporter(file, model).importInto(deck)
    # since it doesn't understand the pending file format,
    # a reverse entry will be generated for everything
    assert len(deck) == 8
    setup1()
    model = stdmodels.byName("Japanese")
    imp = TextImporter(file, model)
    imp.importInto(deck)
    fact = deck.facts.byField("Expression", "to eat")[0]
    # importing is verbatim - it won't try and understand fc pending files
    assert fact['Expression'] == "to eat"
    assert fact['Reading'] == ""
    assert fact['Meaning'] == u"(たべる) 食べる"
    assert len(deck) == 8
    # switch to a single card model and import
    setup1()
    model = stdmodels.byName("Other")
    model.cards = model.cards[0:1]
    TextImporter(file, model).importInto(deck)
    assert len(deck) == 5
    setup1()
    model = stdmodels.byName("Other")
    model.allcards = model.allcards[0:1]
    model.cards = model.allcards
    TextImporter(file, model).importInto(deck)
    assert len(deck) == 5
    setup1()
    model = stdmodels.byName("Other")
    model.allcards = model.allcards[1:2]
    model.cards = model.allcards
    TextImporter(file, model).importInto(deck)
    assert len(deck) == 5

@nose.with_setup(setup1)
def test_importtext4fields():
    file = os.path.join(testDir, "importing/text-4fields.txt")
    model = stdmodels.byName("Other")
    rep = TextImporter(file, model).importInto(deck)
    assert len(deck) == 2

@nose.with_setup(setup1)
def test_importpending():
    file = os.path.join(testDir, "importing/text-2fields.txt")
    model = stdmodels.byName("Japanese")
    rep = FcPendingImporter(file, model).importInto(deck)
    # missing entries are not added, as single detection is on
    assert len(deck) == 5
    assert len(deck.facts) == 3
    # test single models
    setup1()
    model = stdmodels.byName("Japanese")
    model.cards = model.allcards[0:1]
    FcPendingImporter(file, model).importInto(deck)
    assert len(deck) == 5
    setup1()
    model = stdmodels.byName("Japanese")
    model.cards = model.allcards[1:2]
    FcPendingImporter(file, model).importInto(deck)
    assert len(deck) == 5

@nose.with_setup(setup1)
def test_importtriangle():
    file = unicode(os.path.join(testDir, "importing/fleurs.fc"))
    model = stdmodels.byName("Other")
    AnkiV2Importer(file, model).importInto(deck)
    assert len(deck) == 3

@nose.with_setup(setup1)
def test_importtblanktextfield():
    file = unicode(os.path.join(testDir, "importing/text-blank.txt"))
    model = stdmodels.byName("Japanese")
    model.unique = []
    model.required = []
    del model.cards[1]
    rep = TextImporter(file, model).importInto(deck)
    assert not deck.facts.byField("Expression", "foo")[0]["Meaning"]
    assert not deck.facts.byField("Reading", "3")[0]["Expression"]
    assert not deck.facts.byField("Meaning", "5")[0]["Reading"]
