# coding: utf-8

import nose, os, time
import anki
from anki.errors import *
from anki.facts import *
from anki.models import *
from anki.cards import Card

deck = None
ds = None
testDir = os.path.dirname(__file__)

def setup1():
    global deck, ds
    deck = anki.Controller.Deck()
    ds = anki.Controller.DeckStorage()

#
##########################################################################

def addAuto(n=""):
    f = Fact()
    for field in deck.facts.models.current.fields:
        f[field.name] = field.name + n
    deck.addFact(f)

def addBunch():
    for i in range(10):
        addAuto(str(i))

@nose.with_setup(setup1)
def test_sched1():
    addBunch()
    card = deck.sched.getCard()
    card2 = card
    assert card
    assert deck.sched.cardIsNew(card)
    assert deck.sched.cardIsYoung(card)
    assert deck.sched.cardIsInInitialState(card)
    deck.sched.answerCard(card, 0)
    assert not deck.sched.cardIsNew(card)
    assert deck.sched.cardIsInInitialState(card)
    assert deck.sched.cardIsYoung(card)
    deck.sched.answerCard(card, 4)
    assert not deck.sched.cardIsNew(card)
    assert not deck.sched.cardIsInInitialState(card)
    assert deck.sched.cardIsYoung(card)
    deck.sched.answerCard(card, 4)
    assert not deck.sched.cardIsNew(card)
    assert not deck.sched.cardIsYoung(card)
    assert not deck.sched.cardIsInInitialState(card)
    deck.sched.answerCard(card, 0)
    assert not deck.sched.cardIsNew(card)
    assert deck.sched.cardIsYoung(card)
    assert deck.sched.cardIsInInitialState(card)
    # note that the other card in the pair has also become unavailable for now
    assert deck.sched.pendingCards() == (0, 9)
    card.nextTime = time.time()
    assert deck.sched.pendingCards() == (1, 9)
    # finish deck
    n = 0
    while card:
        card = deck.sched.getCard()
        if card:
            deck.sched.answerCard(card, 4)
            n += 1
    assert n == 10
    # pending time strings
    assert deck.sched.pendingCards() == (0, 0)
    assert deck.sched.earliestTime()
    assert deck.sched.earliestTimeStr()
    assert deck.sched.nextCardMsg()
    for i in range(5):
        assert deck.sched.nextIntervalStr(card2, i)

@nose.with_setup(setup1)
def test_copy():
    f = Fact()
    f['Front'] = "foo"; f['Back'] = "bar"
    card = deck.addFact(f)[0]
    card2 = Card(f, card.model)
    assert card.__dict__ != card2.__dict__
    deck.sched.copyToCard(card, card2)
    card2.tags = card.tags
    card2.created = card.created
    card2.id = card.id
    card2.parent = card.parent
    card2.__dict__['modified'] = card.modified
    assert card.__dict__ == card2.__dict__

@nose.with_setup(setup1)
def test_daily():
    assert deck.sched.today() == 0
    deck.created = time.time() - 86400
    assert deck.sched.today() >= 1

@nose.with_setup(setup1)
def test_priorities():
    deck.facts.models.current.cards = deck.facts.models.current.allcards
    f = Fact()
    f.tags.append("fact1")
    f['Front'] = "1"; f['Back'] = "2"
    deck.addFact(f)
    f = Fact()
    f.tags.append("fact2")
    f['Front'] = "3"; f['Back'] = "4"
    deck.addFact(f)
    c1 = [c for c in deck if c.question == "1"][0]; c1.tags.append("card1")
    c2 = [c for c in deck if c.question == "2"][0]; c2.tags.append("card2")
    c3 = [c for c in deck if c.question == "3"][0]; c3.tags.append("card3")
    c4 = [c for c in deck if c.question == "4"][0]; c4.tags.append("card4")
    # set some of our cards as old
    deck.sched.answerCard(c1, 0)
    c1.nextTime = time.time() - 1
    deck.sched.answerCard(c2, 0)
    c2.nextTime = time.time() - 1
    c1.fact.lastCard = None
    # single field
    deck.sched.highPriority.append("card4")
    deck.updatePriorities()
    assert deck.sched.getCards()[0] == c4
    deck.sched.highPriority = []
    deck.sched.lowPriority.append("card4")
    deck.updatePriorities()
    assert deck.sched.getCards()[3] == c4
    # facts should work too
    deck.sched.lowPriority = []
    deck.sched.highPriority.append("fact2")
    deck.updatePriorities()
    cds = deck.sched.getCards()
    assert ((cds[0] == c3 or cds[1] == c3) and
            (cds[0] == c4 or cds[1] == c4))
    # high overrides low
    deck.sched.highPriority = ["card1"]
    deck.sched.lowPriority = ["fact1"]
    deck.updatePriorities()
    cds = deck.sched.getCards()
    assert cds[0] == c1
    assert cds[3] == c2
    # medium vs high priority
    deck.sched.highPriority = []
    deck.sched.lowPriority = []
    deck.updatePriorities()
    deck.remove(c2)
    # we have one old and one new. the old should come first
    assert deck.sched.getCard() == c1
    # if we put c3 in high priority, it'll come first
    deck.sched.highPriority = ["card3"]
    deck.updatePriorities()
    assert deck.sched.getCard() == c3
    # since c3 was created before c4, it'll be first
    deck.sched.highPriority = []
    deck.updatePriorities()
    cds = deck.sched.getCards()
    assert cds[1] == c3 and cds[2] == c4
    # if we put c4 in medium priority, it should come second
    deck.sched.medPriority = ["card4"]
    deck.updatePriorities()
    assert deck.sched.getCards()[1] == c4
